<?php

/**
 * The file that includes all helper functions for DivideBuy.
 * 
 * @since    1.0
 * @package  Divide_Buy
 * @author   Cygnet Infotech Pvt. Ltd.
 */

class DivideBuy_Helper{
    
    public function __construct(){
        add_action('woocommerce_checkout_process', 'create_new_order');
        add_filter('woocommerce_shipping_chosen_method', 'set_shipping_method', 10, 2);
    }
    
    /**
     * 
     * @return dividebuy products count available in cart.
     */
    public function get_dividebuy_products_count(){
        
        $dividebuy_product_count = 0;

        foreach ( WC()->cart->get_cart() as $cart_item_key => $cart_item ){
            $product_id = apply_filters( 'woocommerce_cart_item_product_id', $cart_item['product_id'], $cart_item, $cart_item_key );
            $product_dividebuy_support = get_post_meta( $product_id, 'product_dividebuy_support', true );

            if( $product_dividebuy_support == 'enabled' ){
                $dividebuy_product_count++;
            }
        }
        return $dividebuy_product_count;
    }
    
    /**
     * 
     * @return non dividebuy products count available in cart.
     */
    public function get_nondividebuy_products_count(){
        
        $nondividebuy_product_count = 0;

        foreach ( WC()->cart->get_cart() as $cart_item_key => $cart_item ){
            $product_id = apply_filters( 'woocommerce_cart_item_product_id', $cart_item['product_id'], $cart_item, $cart_item_key );
            $product_dividebuy_support = get_post_meta( $product_id, 'product_dividebuy_support', true );

            if( $product_dividebuy_support == 'disabled' ){
                $nondividebuy_product_count++;
            }
        }
        return $nondividebuy_product_count;
    }
    
    /**
     * 
     * @return dividebuy products grand total available in cart.
     */
    public function get_dividebuy_cart_products_total(){
        
        $dividebuy_grand_total = 0;
        foreach ( WC()->cart->get_cart() as $cart_item_key => $cart_item ){
            //setting the variables
            $discountAmount = 0;
            $discountedVatAmount = 0;
            //Product level discount
            $discountProAmount = $cart_item['line_subtotal'] - $cart_item['line_total'];
            //Vat tax level discount
            $discountedVatAmount = $cart_item['line_subtotal_tax'] - $cart_item['line_tax'];
            //Total Discount amoutn
            $totalDiscount = $discountedVatAmount+$discountProAmount;
            $_product   = apply_filters( 'woocommerce_cart_item_product', $cart_item['data'], $cart_item, $cart_item_key );
            $product_id = apply_filters( 'woocommerce_cart_item_product_id', $cart_item['product_id'], $cart_item, $cart_item_key );
            $product_dividebuy_support = get_post_meta( $product_id, 'product_dividebuy_support', true );
            
            if ( $_product && $_product->exists() && $cart_item['quantity'] > 0 && apply_filters( 'woocommerce_cart_item_visible', true, $cart_item, $cart_item_key ) && $product_dividebuy_support == 'enabled' ){
                $dividebuy_grand_total = $dividebuy_grand_total + ($_product->get_price_including_tax() * $cart_item['quantity'])-$totalDiscount;
            }
        }
        return number_format($dividebuy_grand_total,2);
    }
    
    /**
     * 
     * @return required minimum order amount at time of checkout
     */
    public function check_minimum_order_amount(){
        $minimum_order_amount_status = "true";
        $order_minimum_value = str_replace(",", "", get_option( 'wc_settings_tab_dividebuy_minimum_order_value' ));
        $order_total = str_replace(",", "", $this->get_dividebuy_cart_products_total());
        if( $order_total < $order_minimum_value ){
            $minimum_order_amount_status = "false";
        }
        return array(
            'status' => $minimum_order_amount_status,
            'min_order_amount' => $order_minimum_value
        );
    }
    
    public function get_instalments_price($price){
        $instalments = unserialize(get_option('wc_settings_tab_dividebuy_retailer_installments'));
        
        foreach ( $instalments as $instalment ){
            $instalment_price = $price/$instalment['key'];
            if( $price > $instalment['value']){
                echo $instalment['key'] . ' monthly instalments of ' . number_format($instalment_price, 2). ' - Yes<br/>';
            }else{
                echo $instalment['key'] . ' monthly instalments of ' . number_format($instalment_price, 2).' - No<br/>';
            }
        };
    }

    /**
     * This function removes all products from cart and stores it in one session.
     */
    public function remove_products_from_cart(){
        $items = WC()->cart->get_cart();
        $non_dividebuy_cart_products = array();
        $i=0;
        
        foreach($items as $item => $values) {
            $product_quantity = $values['quantity'];
            $product_id = $values['product_id'];
            $product_variation_id = $values['variation_id'];            
            
            if($product_variation_id){
                $product_dividebuy_support = get_post_meta( $product_id, 'product_dividebuy_support', true );
                $cart_id = WC()->cart->generate_cart_id($product_id, $product_variation_id);
            }else{
                $product_dividebuy_support = get_post_meta( $product_id, 'product_dividebuy_support', true );
                $cart_id = WC()->cart->generate_cart_id($product_id);
            }
            
            // Adding non DivideBuy products in array then removing from cart.
            $non_dividebuy_cart_products[$i]['product_id'] = $product_id;
            $non_dividebuy_cart_products[$i]['variation_id'] = $product_variation_id;
            $non_dividebuy_cart_products[$i]['product_quantity'] = $product_quantity;
            WC()->cart->set_quantity( $item, 0, true  );
            $i++;
        }
        
        // Creating session for storing non DivideBuy product data.
        if(WC()->session->get('nondividebuy_products_cart_session')){
            $existing_cart_session = WC()->session->get('nondividebuy_products_cart_session');
            $new_cart_session = array_merge($non_dividebuy_cart_products, $existing_cart_session);
            WC()->session->set( 'nondividebuy_products_cart_session', $new_cart_session );
        }else{
            WC()->session->set( 'nondividebuy_products_cart_session', $non_dividebuy_cart_products );
        }
    }
    
    /**
     * 
     * Function for removing non DivideBuy products from cart when user will continue in cart page modal.
     */
    public function remove_nondividebuy_products_from_cart(){
        
        $items = WC()->cart->get_cart();
        $non_dividebuy_cart_products = array();
        $i=0;
        
        foreach($items as $item => $values) {
            $product_quantity = $values['quantity'];
            $product_id = $values['product_id'];
            $product_variation_id = $values['variation_id'];            
            
            if($product_variation_id){
                $product_dividebuy_support = get_post_meta( $product_id, 'product_dividebuy_support', true );
                $cart_id = WC()->cart->generate_cart_id($product_id, $product_variation_id);
            }else{
                $product_dividebuy_support = get_post_meta( $product_id, 'product_dividebuy_support', true );
                $cart_id = WC()->cart->generate_cart_id($product_id);
            }
            
            // Adding non DivideBuy products in array then removing from cart.
            if($product_dividebuy_support == "disabled"){
                $non_dividebuy_cart_products[$i]['product_id'] = $product_id;
                $non_dividebuy_cart_products[$i]['variation_id'] = $product_variation_id;
                $non_dividebuy_cart_products[$i]['product_quantity'] = $product_quantity;
                WC()->cart->set_quantity( $item, 0, true  );
                $i++;
            }
        }
        
        // Creating session for storing non DivideBuy product data.
        if(WC()->session->get('nondividebuy_products_cart_session')){
            $existing_cart_session = WC()->session->get('nondividebuy_products_cart_session');
            $new_cart_session = array_merge($non_dividebuy_cart_products, $existing_cart_session);
            WC()->session->set( 'nondividebuy_products_cart_session', $new_cart_session );
        }else{
            WC()->session->set( 'nondividebuy_products_cart_session', $non_dividebuy_cart_products );
        }
    }
    
    /**
     * 
     * @return current package details for shipping.
     */
    public function get_shipping_packages($postcode){
        
        define( 'WOOCOMMERCE_CHECKOUT', true );
        define( 'WOOCOMMERCE_CART', true );

        $country = wc_clean("GB");

        if ( $country ) {
            WC()->customer->set_location( $country, $state, $postcode, $city );
            WC()->customer->set_shipping_location( $country, $state, $postcode, $city );
        }

        WC()->customer->calculated_shipping( true );

        do_action( 'woocommerce_calculated_shipping' );
        WC()->cart->calculate_shipping();
        $packages = WC()->shipping->get_packages();
        $packages['message'] = "";
        
        // Checking non deliverable postcodes for shipping.
        $postcode = str_replace(' ', '', $postcode);  
        $postcodes = get_option('wc_settings_tab_dividebuy_checkout_postcode_list');
        $dividebuy_postcodes = explode(",", $postcodes);
        $is_deliverable = "true";
        foreach($dividebuy_postcodes as $code){
            $code = str_replace(' ', '', $code);  
            if(strtolower(trim($code)) == strtolower(trim($postcode))){
                $is_deliverable = "false";
                break;
            }
        }
        //check retailor is allowed for auto checkout if they do not have any shipping method
        $shipping_auto = get_option('wc_settings_tab_auto_shipping');
        if($shipping_auto == 1 && count($packages[0]['rates']) == 0 && $is_deliverable != "false")
        {
            $packages = "";
            $packages['allow'] = 1;
        }
        
        // Check if there are no shipping methods available.
        if(count($packages[0]['rates']) == 0 && $shipping_auto == 0){
            $packages = "";
            $packages['message'] = "There are no shipping methods available. Please double check your address, or contact us if you need any help.";
            $packages['allow'] = 0;
        }
        
        // Check for postcodes in which DivideBuy shipping is disabled.
        if($is_deliverable == "false"){
            $packages = "";
            $packages['message'] = "Unfortunately we are unable to deliver to this postcode. Please call our Customer Service team for more details: 0800 085 0885.";
            $packages['allow'] = 0;
        }
        
        return $packages;
    }

    /**
     * This function retrieves shipping packages based on order items and returns if shipping value has been changed or not.
     */
    public function get_order_items_shipping_packages($order){
        $shipping_changed = "false";
        $order_item = $order->get_items();
        
        foreach ($order_item as $product){
            $product_id = $product['product_id'];
            $product_variation_id = $product['variation_id'];
            
            // Check if product has variation.
            if($product_variation_id){
                $variation = wc_get_product($product_variation_id);
                $attributes = $variation->get_variation_attributes();
                WC()->cart->add_to_cart($product_id, $product['qty'], $product_variation_id, $attributes);
            }else{
                WC()->cart->add_to_cart($product_id, $product['qty']);
            }
        }

        // Getting order shipping information.
        $shipping_data = $order->get_items('shipping');
        foreach($shipping_data as $shipping){
            $shipping_label = $shipping['name'];
            $shipping_method = $shipping['method_id'];
            $shipping_cost = $shipping['cost'];
            $shipping_incl_vat = $shipping['cost'] + $shipping['total_tax'];
        }

        $packages = $this->get_shipping_packages($user_postcode);
        $shipping_packages = $packages[0]['rates'];

        foreach ( $shipping_packages as $package ){
            $package_method_id = explode(":", $package->method_id);
            $shipping_method = explode(":", $shipping_method);
            if($package_method_id[0] == $shipping_method[0]){

                if(wc_format_decimal($package->cost) != $shipping_cost){
                    $shipping_changed = "true";
                }
                break;
            }
        }

        return $shipping_changed;
    }
    
    /**
     * 
     * Function for placing new order when user will checkout from cart page DivideBuy modal.
     */
    public function create_new_order(){
        
        $non_dividebuy_products_count = DivideBuy_Helper::get_nondividebuy_products_count();
        if($non_dividebuy_products_count == 0){
            $postcode = filter_input(INPUT_POST, 'user_postcode');
            $shipping_method = filter_input(INPUT_POST, 'shipping_method');
            $packages = DivideBuy_Helper::get_shipping_packages($postcode);
            $shipping_packages = $packages[0]['rates'];
            foreach($shipping_packages as $package){
                if($package->method_id == $shipping_method){
                    $shipping_details = $package;
                }
            }
            
            $user_id = get_current_user_id();
            $customer = get_userdata($user_id);
            $customer_address = new WC_Customer($user_id);
            
            if($user_id){
                $order_data = array(
                    'customer_id' => $user_id
                );
            }
            if(is_user_logged_in()){
                $address = array(
                    'first_name' => get_user_meta( $user_id, 'shipping_first_name', true ),
                    'last_name'  => get_user_meta( $user_id, 'shipping_last_name', true ),
                    'email'      => $customer->user_email,
                    'phone'      => get_user_meta( $user_id, 'billing_phone', true ),
                    'address_1'  => $customer_address->shipping_address_1,
                    'address_2'  => $customer_address->shipping_address_2,
                    'city'       => $customer_address->city,
                    'county'     => $customer_address->shipping_state,
                    'postcode'   => $postcode,
                    'country'    => $customer_address->shipping_country
                );                    
            }else{
                $address = array(
                    'first_name' => 'First Name',
                    'last_name'  => 'Last Name',
                    'email'      => '',
                    'phone'      => '987-654-3210',
                    'address_1'  => '123 Main st.',
                    'address_2'  => '104',
                    'city'       => 'London',
                    'county'      => 'Wales',
                    'postcode'   => $postcode,
                    'country'    => 'GB'
                );
            }

            try{
                // Now we create the order
                $order = wc_create_order($order_data);

                foreach (WC()->cart->get_cart() as $cart_item_key => $values) {
                    $order->add_product(
                        $values['data'], $values['quantity'], array(
                            'variation' => $values['variation'],
                            'totals' => array(
                                'subtotal' => $values['line_subtotal'],
                                'subtotal_tax' => $values['line_subtotal_tax'],
                                'total' => $values['line_total'],
                                'tax' => $values['line_tax'],
                                'tax_data' => $values['line_tax_data']
                            )
                        )
                    );
                }

                $order->set_address( $address, 'billing' );
                $order->set_address( $address, 'shipping' );
                $order->update_status('freeze-dividebuy', __( 'Waiting for confirmation from DivideBuy.', 'woocommerce' ));
                update_post_meta( $order->id, '_payment_method', 'dividebuy' );
                update_post_meta( $order->id, '_payment_method_title', 'DivideBuy' );
                update_post_meta( $order->id, 'order_is_dividebuy', 'yes' );

                $item_id = wc_add_order_item( $order->id, array(
                    'order_item_name'       => $shipping_details->label,
                    'order_item_type'       => 'shipping'
                ) );

                if ( $item_id ) {
                    wc_add_order_item_meta( $item_id, 'method_id', $shipping_details->method_id );
                    wc_add_order_item_meta( $item_id, 'cost', wc_format_decimal($shipping_details->cost) );
                    wc_add_order_item_meta( $item_id, 'taxes', $shipping_details->taxes );
                }

                $order->calculate_taxes();
                $order->calculate_totals();
                $order_id = $order->id;

                $url = Divide_Buy_Configuration::get_order_redirect_url();
                $splashKey = base64_encode($order_id.':'.get_option('wc_settings_tab_retailer_id'));
                $redirect_url = $url.'?splashKey='.$splashKey;
                global $woocommerce;
               /* $woocommerce->cart->empty_cart();*/
                DivideBuy_Helper::add_session_products_to_cart();
                $woocommerce->session->__unset( 'nondividebuy_products_cart_session' );
                echo $redirect_url;

            } catch (Exception $ex) {
                DivideBuy_Helper::add_session_products_to_cart();
            }
        }else{
            echo $woocommerce->cart->get_cart_url();
        }
        
    }
    
    /**
     * 
     * Function for adding session products in cart after placing order.
     */
    public function add_session_products_to_cart(){
        $session_products = WC()->session->get('nondividebuy_products_cart_session');
        foreach ($session_products as $product){
            $product_id = $product['product_id'];
            $product_variation_id = $product['variation_id'];
            
            if($product_variation_id){
                $variation = wc_get_product($product_variation_id);
                $attributes = $variation->get_variation_attributes();
                WC()->cart->add_to_cart($product_id, $product['product_quantity'], $product_variation_id, $attributes);
            }else{
                WC()->cart->add_to_cart($product_id, $product['product_quantity']);
            }
        }
    }
    
    /**
     * 
     * @param type $order
     * @return string for product stock status
     */
    public function check_order_product_stock($order){
        $stock_available = "true";
        $order_item = $order->get_items();
        
        foreach( $order_item as $product ) {
            if($product['variation_id']){
                $product_id = $product['variation_id'];
            }else{
                $product_id = $product['product_id'];
            }
            
            $manage_stock = get_post_meta( $product_id, '_manage_stock', true );
            $stock = get_post_meta( $product_id, '_stock', true );
            $stock_status = get_post_meta( $product_id, '_stock_status', true );
            if($manage_stock == "yes"){
                if($stock < $product['qty'] || $stock_status == "outofstock"){
                    $stock_available = "false";
                    break;
                }
            }
        }        
        return $stock_available;
    }
    
    /**
     * 
     * @param type $order
     * creates duplicate order based on provided order details.
     */
    public function create_duplicate_order($order){
        $order_data = array(
            'customer_id' => $order->post->post_author
        );
        $current_order_id = $order->id;
        $new_order = wc_create_order($order_data);
        $new_order_id = $new_order->id;
        
        update_post_meta( $new_order_id, '_order_shipping',         get_post_meta($current_order_id, '_order_shipping', true) );
        update_post_meta( $new_order_id, '_order_discount',         get_post_meta($current_order_id, '_order_discount', true) );
        update_post_meta( $new_order_id, '_cart_discount',          get_post_meta($current_order_id, '_cart_discount', true) );
        update_post_meta( $new_order_id, '_order_tax',              get_post_meta($current_order_id, '_order_tax', true) );
        update_post_meta( $new_order_id, '_order_shipping_tax',     get_post_meta($current_order_id, '_order_shipping_tax', true) );
        update_post_meta( $new_order_id, '_order_total',            get_post_meta($current_order_id, '_order_total', true) );
        
        update_post_meta( $new_order_id, '_shipping_country',       get_post_meta($current_order_id, '_shipping_country', true));
        update_post_meta( $new_order_id, '_shipping_first_name',    get_post_meta($current_order_id, '_shipping_first_name', true));
        update_post_meta( $new_order_id, '_shipping_last_name',     get_post_meta($current_order_id, '_shipping_last_name', true));
        update_post_meta( $new_order_id, '_shipping_company',       get_post_meta($current_order_id, '_shipping_company', true));
        update_post_meta( $new_order_id, '_shipping_address_1',     get_post_meta($current_order_id, '_shipping_address_1', true));
        update_post_meta( $new_order_id, '_shipping_address_2',     get_post_meta($current_order_id, '_shipping_address_2', true));
        update_post_meta( $new_order_id, '_shipping_city',          get_post_meta($current_order_id, '_shipping_city', true));
        update_post_meta( $new_order_id, '_shipping_state',         get_post_meta($current_order_id, '_shipping_state', true));
        update_post_meta( $new_order_id, '_shipping_postcode',      get_post_meta($current_order_id, '_shipping_postcode', true));
        
        update_post_meta( $new_order_id, '_billing_country',       get_post_meta($current_order_id, '_billing_country', true));
        update_post_meta( $new_order_id, '_billing_first_name',    get_post_meta($current_order_id, '_billing_first_name', true));
        update_post_meta( $new_order_id, '_billing_last_name',     get_post_meta($current_order_id, '_billing_last_name', true));
        update_post_meta( $new_order_id, '_billing_company',       get_post_meta($current_order_id, '_billing_company', true));
        update_post_meta( $new_order_id, '_billing_address_1',     get_post_meta($current_order_id, '_billing_address_1', true));
        update_post_meta( $new_order_id, '_billing_address_2',     get_post_meta($current_order_id, '_billing_address_2', true));
        update_post_meta( $new_order_id, '_billing_city',          get_post_meta($current_order_id, '_billing_city', true));
        update_post_meta( $new_order_id, '_billing_state',         get_post_meta($current_order_id, '_billing_state', true));
        update_post_meta( $new_order_id, '_billing_postcode',      get_post_meta($current_order_id, '_billing_postcode', true));
        
        update_post_meta( $new_order_id, 'order_is_dividebuy',      get_post_meta($current_order_id, 'order_is_dividebuy', true));
        update_post_meta( $new_order_id, 'order_tracking_number',      get_post_meta($current_order_id, 'order_tracking_number', true));
        
        foreach($order->get_items() as $original_order_Item){
            $item_name = $original_order_Item['name'];
            $qty = $original_order_Item['qty'];
            $line_total = $original_order_Item['line_total'];
            $product_id = $original_order_Item['product_id'];

            $item_id = wc_add_order_item( $new_order_id, array(
                'order_item_name'       => $item_name,
                'order_item_type'       => 'line_item'
            ) );

            wc_add_order_item_meta( $item_id, '_qty', $qty );
            wc_add_order_item_meta( $item_id, '_product_id', $product_id );
            wc_add_order_item_meta( $item_id, '_line_subtotal', wc_format_decimal( $line_total ) );
            wc_add_order_item_meta( $item_id, '_line_total', wc_format_decimal( $line_total ) );
            wc_add_order_item_meta( $item_id, '_line_tax', wc_format_decimal( '0' ) );
            wc_add_order_item_meta( $item_id, '_line_subtotal_tax', wc_format_decimal( '0' ) );
        }
        
        $original_order_shipping_items = $order->get_items('shipping');
        foreach ( $original_order_shipping_items as $original_order_shipping_item ) {
            $item_id = wc_add_order_item( $new_order_id, array(
                'order_item_name'       => $original_order_shipping_item['name'],
                'order_item_type'       => 'shipping'
            ) );

            if ( $item_id ) {
                wc_add_order_item_meta( $item_id, 'method_id', $original_order_shipping_item['method_id'] );
                wc_add_order_item_meta( $item_id, 'cost', wc_format_decimal( $original_order_shipping_item['cost'] ) );
                wc_add_order_item_meta( $item_id, 'taxes', unserialize($original_order_shipping_item['taxes']) );
            }
        }
        
        $original_order_coupons = $order->get_items('coupon');
        foreach ( $original_order_coupons as $original_order_coupon ) {
            $item_id = wc_add_order_item( $new_order_id, array(
                'order_item_name'       => $original_order_coupon['name'],
                'order_item_type'       => 'coupon'
            ) );
            // Add line item meta
            if ( $item_id ) {
                wc_add_order_item_meta( $item_id, 'discount_amount', $original_order_coupon['discount_amount'] );
            }
        }
        
        $new_order->calculate_taxes();
        $new_order->calculate_shipping();
        $new_order->calculate_totals();
        
        //Payment Information
        update_post_meta( $new_order_id, '_payment_method',         get_post_meta($current_order_id, '_payment_method', true) );
        update_post_meta( $new_order_id, '_payment_method_title',   get_post_meta($current_order_id, '_payment_method_title', true) );
        update_post_meta( $new_order_id, 'Transaction ID',          get_post_meta($current_order_id, 'Transaction ID', true) );
        $new_order->payment_complete();
        
        $new_order->update_status('processing');
        
        return $new_order->id;
    }
    
    /**
     * 
     * @param type $retailer_id
     * @param type $post_id
     * @param type $order_tracking_number
     * @param type $store_token
     * @param type $store_authentication
     * 
     * function for sending order tracking numbers to laravel.
     */
    public function send_tracking_number_information($post_id, $order_tracking_number, $carrier_code, $delete_tracking){
        $retailer_id = get_option('wc_settings_tab_retailer_id');
        $store_token = get_option('wc_settings_tab_retailer_token_number');
        $store_authentication = get_option('wc_settings_tab_retailer_authentication_number');
        $order = new WC_Order( $post_id );
        $order_item = $order->get_items();
        $product_details = array();
        $tracking_information = array();
        $product_count = 0;
        
        foreach ($order_item as $product){
            $product_id = $product['product_id'];
            $product_variation_id = $product['variation_id'];
            
            // Check if product has variation.
            if ($product_variation_id) {
                $single_product_data = new WC_Product_Variation($product_variation_id);
                $product_image = wp_get_attachment_image_src( get_post_thumbnail_id( $product_variation_id ), 'single-post-thumbnail' );
            } else {
                $single_product_data = new WC_Product($product_id);
                $product_image = wp_get_attachment_image_src( get_post_thumbnail_id( $product_id ), 'single-post-thumbnail' );
            }
            
            $product_details[$product_count]['sku'] = $single_product_data->get_sku();
            $product_details[$product_count]['qty'] = $product['qty'];
        }
        
        $tracking_information[0]['trackNumber'] = $order_tracking_number;
        $tracking_information[0]['carrierCode'] = $carrier_code;
        $tracking_information[0]['title'] = $carrier_code;
        
        // Array for sending all information of tracking.
        $postdata_array = json_encode(array(
            "retailerId"            => $retailer_id,
            "storeOrderId"          => $post_id,
            "storeToken"            => $store_token,
            "storeAuthentication"   => $store_authentication,
            "deleteTracking"        => $delete_tracking,
            "trackingInfo"          => $tracking_information,
            "productDetails"        => $product_details
        ));
        return wp_remote_retrieve_body(wp_remote_post(Divide_Buy_Configuration::get_api_url().'api/tracking' , array( 'headers' => array('content-type' => 'application-json'), 'body'=> $postdata_array, 'method' => 'POST')));
    }
    /* Check current ip and given ip are same or not*/
    function allowip_check()
    {
        $server_ip = $_SERVER['REMOTE_ADDR'];
        $allowedip = get_option( 'wc_settings_tab_allowed_ip' );
        $allowedip_array = explode(",", $allowedip);
        if(!empty($allowedip_array) && $allowedip !='')
        {
            if(in_array($server_ip,$allowedip_array))
            {
                $result_ip = true;
            }
            else
            {
                $result_ip = false;
            }
        }
        else
        {

            $result_ip = true;
        }
        return $result_ip;
    }

}
