<?php

/**
 * This file includes all the functions related to all checkout functions.
 */

if ( ! defined( 'ABSPATH' ) ) { 
    exit; // Exit if file accessed directly
}

/**
 * Adding function for displaying address note after billing address fields.
 */

/**
 * 
 * @param type $available_gateways
 * @return type
 * 
 * If there are no products in cart which are DivideBuy supported then
 * DivideBuy payment method will get hidden in checkout page.
 */

// Disable DivideBuy payment gateway if there are no DivideBuy products in cart.
function dividebuy_payment_gateway_disable( $available_gateways ) {
    if(!is_admin()){
        $dividebuy_helper = new DivideBuy_Helper();
        $dividebuy_product_count = $dividebuy_helper->get_dividebuy_products_count();
        $DivideBuy_Helper = new DivideBuy_Helper();
        $result_ip = $DivideBuy_Helper->allowip_check();
        if (($dividebuy_product_count == 0 || $result_ip == false) && is_checkout()) {
            unset( $available_gateways['dividebuy'] );
        }
    }    
    return $available_gateways;
}

add_filter( 'woocommerce_available_payment_gateways', 'dividebuy_payment_gateway_disable' );

// Displaying shipping address note.
function dividebuy_billing_address_note() {
    $dividebuy_helper = new DivideBuy_Helper();
    $dividebuy_product_count = $dividebuy_helper->get_dividebuy_products_count();
    $available_gateways = WC()->payment_gateways->get_available_payment_gateways();

    if ( $dividebuy_product_count != 0 && isset($available_gateways["dividebuy"]) && !empty($available_gateways["dividebuy"])){
        echo '<div id="dividebuy_address_note">';
        wc_print_notice( __( 'For security reasons, your billing and delivery addresses must match if choosing Interest-Free Credit with DivideBuy.', 'woocommerce' ), 'notice' );
        echo '</div>';
    }
}
add_action( 'woocommerce_before_checkout_shipping_form', 'dividebuy_billing_address_note' );

// Displaying modal on payment method click if there are mixed products in cart.
add_action('woocommerce_after_checkout_form', 'dividebuy_checkout_modal');
function dividebuy_checkout_modal(){
    $dividebuy_helper = new DivideBuy_Helper();
    $store_name = get_option('wc_settings_tab_retailer_name');
    $order_minimum_value = $dividebuy_helper->check_minimum_order_amount();
    $cross_icon = plugins_url() . '/dividebuy/assets/images/closebtn.png';
    ?>
    <!-- Modal -->
    <div id="dividebuy_modal" class="modal fade custom-modal">
        <div class="modal-dialog">

            <!-- Modal content-->
            <div class="modal-content">

                <div class="modal-header clearfix">
                    <a href="#" id="scroll"  data-dismiss="modal" class="close"><img src="<?php echo $cross_icon; ?>" /></button></a>
                </div>
                <div class="modal-body clearfix">

                    <div id="div_login_form">
                        <div>
                            <div class="">
                                <p class="text-left blue-text cart-heading">
                                    <?php esc_html_e( 'The following products in your basket are currently eligible for Interest-Free Credit:', 'woocommerce'); ?>
                                </p>
                                <div class="col-sm-7">
                                    <div class="row">
                                        <div class="shoppingcartTable">
                                            <div class="shopping-scroll">
                                                <ul class="datas">
                                                    <?php
                                                    foreach ( WC()->cart->get_cart() as $cart_item_key => $cart_item ){
                                                        $_product   = apply_filters( 'woocommerce_cart_item_product', $cart_item['data'], $cart_item, $cart_item_key );
                                                        $product_id = apply_filters( 'woocommerce_cart_item_product_id', $cart_item['product_id'], $cart_item, $cart_item_key );
                                                        $product_dividebuy_support = get_post_meta( $product_id, 'product_dividebuy_support', true );

                                                        if ( $_product && $_product->exists() && $cart_item['quantity'] > 0 && apply_filters( 'woocommerce_cart_item_visible', true, $cart_item, $cart_item_key ) && $product_dividebuy_support == 'enabled' ){
                                                            $product_image_url = wp_get_attachment_image_src(get_post_thumbnail_id($_product->get_id()), 'single-post-thumbnail');
                                                            if($product_image_url == ""){
                                                                $product_image_url = wp_get_attachment_image_src(get_post_thumbnail_id($product_id), 'single-post-thumbnail');
                                                            }


                                                     ?>

                                                            <li class="clearfix">
                                                                <div class="row">
                                                                    <div class="product-image-1 col-sm-4 col-xs-5"><img class="img-responsive img-thumbnail" src="<?php echo $product_image_url[0];?>"/></div>
                                                                    <div class="modal-content-area col-sm-7"><div class="pro-name"><?php echo $_product->get_title();?></div><span class="price"><?php echo get_woocommerce_currency_symbol().$_product->get_price();?></span></div>
                                                                </div>
                                                            </li>
                                                        <?php
                                                        }
                                                    }
                                                    ?>
                                                </ul>
                                            </div>
                                        </div>
                                    </div>
                                </div>
                                <div class="col-sm-4 side-content col-sm-offset-1">
                                    <div class="row">
                                        <p><?php esc_html_e( 'To Spread the Cost Interest-Free on these products, please click the button below.', 'woocommerce' ); ?></p>
                                        <div class="sp-methods-divide">
                                            <a href="#" id="btn_proceed_to_checkout" class="btn-blue custom-blue-btn"><?php esc_html_e( 'Continue', 'woocommerce' ); ?></a>
                                            <span class="blue-text grand-total"><?php esc_html_e( 'Grand Total : ', 'woocommerce' ) ?><?php echo get_woocommerce_currency_symbol().$dividebuy_helper->get_dividebuy_cart_products_total();?></span>
                                        </div>
                                    </div>
                                </div>
                                <div class="col-md-12 col-sm-12">
                                    <div class="row">
                                        <div class="shoppingcartTable">
                                            <div class="text-left text-details">
                                                <?php
                                                esc_html_e( 'The remaining products in your basket are not available for Interest-Free Credit.', 'woocommerce' );
                                                esc_html_e( 'You can purchase them directly through '.$store_name.'.', 'woocommerce' );
                                                ?>
                                            </div>
                                            <div class="col-sm-7">
                                                <div class="row">
                                                    <div class="shopping-scroll">
                                                        <ul class="datas">
                                                            <?php
                                                            foreach ( WC()->cart->get_cart() as $cart_item_key => $cart_item ){
                                                                $_product   = apply_filters( 'woocommerce_cart_item_product', $cart_item['data'], $cart_item, $cart_item_key );
                                                                $product_id = apply_filters( 'woocommerce_cart_item_product_id', $cart_item['product_id'], $cart_item, $cart_item_key );
                                                                $product_dividebuy_support = get_post_meta( $product_id, 'product_dividebuy_support', true );

                                                                if ( $_product && $_product->exists() && $cart_item['quantity'] > 0 && apply_filters( 'woocommerce_cart_item_visible', true, $cart_item, $cart_item_key ) && $product_dividebuy_support == 'disabled' ){
                                                                    $product_image_url = wp_get_attachment_image_src(get_post_thumbnail_id($_product->get_id()), 'single-post-thumbnail');
                                                                    if($product_image_url == ""){
                                                                        $product_image_url = wp_get_attachment_image_src(get_post_thumbnail_id($product_id), 'single-post-thumbnail');
                                                                    }

                                                            ?>
                                                                    <li class="clearfix">
                                                                        <div class="row">
                                                                            <div class="product-image-1 col-sm-4 col-xs-5"><img class="img-responsive img-thumbnail" src="<?php echo $product_image_url[0];?>"/></div>
                                                                            <div class="modal-content-area col-sm-7"><div class="pro-name"><?php echo $_product->get_title();?></div><span class="price"><?php echo get_woocommerce_currency_symbol().$_product->get_price();?></span></div>
                                                                        </div>
                                                                    </li>
                                                                <?php
                                                                }
                                                            }
                                                            ?>
                                                        </ul>
                                                    </div>
                                                </div>
                                            </div>
                                        </div>
                                    </div>
                                </div>
                                <div class="clearfix"></div>
                            </div>
                        </div>
                        <div id="splitcost_cover"></div>                         
                    </div>
                    <div id="shipping-loader" class="loader-set" style="margin:0;display: none">
                        <div id="loader"></div>
                    </div>
                    <div class="all_shipping_methods">                           
                    </div>
                </div>
            </div>
        </div>
    </div>
    <?php if( $order_minimum_value['status'] == "false" ){ ?>
        <style>
            
        </style>
        <script>
           jQuery(document).bind('DOMNodeInserted', function(event) {                
                    jQuery('#payment_method_dividebuy').attr('disabled',true);     
                    jQuery(document).find('.payment_method_dividebuy').css('opacity', 0.4);                       
                });;
            jQuery(document).on('mousemove', ".payment_method_dividebuy" , function(e) {
                mousemovebtn(e, this)
            });
            jQuery(document).on({
                mouseenter: function () {
                    // Hover code
                    var content = "<div class='slip_tooltip_container grey'><div class='slip_tooltip_content'><span class='tooltip-box'><span class='blue'>DivideBuy - The easy Interest-Free Checkout.</span> Spend a minimum of <?php echo get_woocommerce_currency_symbol().$order_minimum_value['min_order_amount']; ?> to be eligible to Spread the Cost, Interest-Free using our <span class='blue'>secure platform</span> to fill out our <span class='blue'>60 second application</span>. With us you'll get an <span class='blue'>instant decision. 96% acceptance rate</span> and a <span class='blue'>choice of instalment options</span> to suit you.</span></div></div>";
                    if (!(jQuery('body').find('.slip_tooltip_container').length)) {
                        jQuery(content).appendTo('body').fadeIn('slow');
                    }
                },
                mouseleave: function () {
                    // Hover out code
                    jQuery('.slip_tooltip_container').remove();
                }
            }, '.payment_method_dividebuy');
        </script>
    <?php } elseif( WC()->cart->get_coupons() && get_option('dividebuy_enable_coupon_code') == 'no' ){ ?>
        <script>
           jQuery(document).bind('DOMNodeInserted', function(event) {                
                    jQuery('#payment_method_dividebuy').attr('disabled',true);  
                    jQuery(document).find('.payment_method_dividebuy').css('opacity', 0.4);                          
                });
            jQuery(document).on('mousemove', ".payment_method_dividebuy" , function(e) {
                mousemovebtn(e, this)
            });
            jQuery(document).on({
                mouseenter: function () {
                    // Hover code
                    var content = "<div class='slip_tooltip_container coupon-box'><div class='slip_tooltip_content'><span class='tooltip-box'>This coupon code cannot be used in conjunction with Interest-Free Credit.</span></div></div>";
                    if (!(jQuery('body').find('.slip_tooltip_container').length)) {
                        jQuery(content).appendTo('body').fadeIn('slow');
                    }
                },
                mouseleave: function () {
                    // Hover out code
                    jQuery('.slip_tooltip_container').remove();
                }
            }, '.payment_method_dividebuy');
        </script>

    <?php } ?>
    <script>
        select_default_payment_method();
        jQuery(document).ready(function(){
            jQuery('#scroll').on('click', function () {
                select_default_payment_method();
            });
        });
        jQuery(function(){
            jQuery( 'body' )
            .on( 'updated_checkout', function() {                
                jQuery('input[name="payment_method"]').click(function(){
                    jQuery.ajax({
                        type: 'post',
                        url: '<?php echo get_site_url().'/dividebuy/modal/get_nondividebuy_count'; ?>',
                        success: function(response){
                            if(response > 0){
                                check_cart_mixed_products();
                            }else{
                                jQuery("#place_order").prop("disabled", false);
                                jQuery("#place_order").css("opacity", "1.0");
                            }
                        }
                    });
                });
            });
        });

        jQuery('#scroll').on('click', function () {
            jQuery('#dividebuy_modal').removeClass('in-modal');
            jQuery('body').removeClass('db-modal');
            setTimeout(function() { jQuery("#dividebuy_modal").css("display","none"); }, 500);
        });
        
        jQuery("#btn_proceed_to_checkout").click(function(){
            jQuery("#shipping-loader").show();
            var url = '<?php echo get_site_url().'/dividebuy/modal/shipping_information'; ?>';
            display_shipping_modal(url);
            return false;
        });
        
        jQuery("#billing_postcode").keyup(function () {
            var newPostCode = checkPostCode(document.getElementById('billing_postcode').value);
            if (newPostCode) {
                document.getElementById('billing_postcode').value = newPostCode;
            }
        });
        jQuery("#shipping_postcode").keyup(function () {
            var newPostCode = checkPostCode(document.getElementById('shipping_postcode').value);
            if (newPostCode) {
                document.getElementById('shipping_postcode').value = newPostCode;
            }
        });
        
        jQuery('.shopping-scroll').mCustomScrollbar({ 
            theme:"dark-3"        
        });
    </script>
    <?php
}

add_action('woocommerce_applied_coupon','appliedcoupon');
function appliedcoupon()
{
    $min_order = get_option('wc_settings_tab_dividebuy_minimum_order_value');
    global $woocommerce;
    $dividebuy_grand_total = 0;
        foreach ( WC()->cart->get_cart() as $cart_item_key => $cart_item ){
            $_product   = apply_filters( 'woocommerce_cart_item_product', $cart_item['data'], $cart_item, $cart_item_key );
            $product_id = apply_filters( 'woocommerce_cart_item_product_id', $cart_item['product_id'], $cart_item, $cart_item_key );
            $product_dividebuy_support = get_post_meta( $product_id, 'product_dividebuy_support', true );
            
            if ( $_product && $_product->exists() && $cart_item['quantity'] > 0 && apply_filters( 'woocommerce_cart_item_visible', true, $cart_item, $cart_item_key ) && $product_dividebuy_support == 'enabled' ){
                $dividebuy_grand_total = $dividebuy_grand_total + ($_product->get_price_including_tax() * $cart_item['quantity']);
            }
        }
    $cart_total_amount = $dividebuy_grand_total;
    $enable_dsiable = get_option('dividebuy_enable_coupon_code');
    if($min_order > $cart_total_amount)
    {
    ?>
            <script>
                jQuery(document).bind('DOMNodeInserted', function(event) {     
                jQuery(document).find('.payment_method_dividebuy').css('opacity', 0.4);                
                    jQuery('#payment_method_dividebuy').attr('disabled',true);            
                });
                jQuery(document).on('mousemove', ".payment_method_dividebuy" , function(e) {
                    mousemovebtn(e, this)
                });
                jQuery(document).on({
                    mouseenter: function () {
                        // Hover code
                        var content = "<div class='slip_tooltip_container grey'><div class='slip_tooltip_content'><span class='tooltip-box'><span class='blue'>DivideBuy - The easy Interest-Free Checkout.</span> Spend a minimum of <?php echo get_woocommerce_currency_symbol().$order_minimum_value['min_order_amount']; ?> to be eligible to Spread the Cost, Interest-Free using our <span class='blue'>secure platform</span> to fill out our <span class='blue'>60 second application</span>. With us you'll get an <span class='blue'>instant decision. 96% acceptance rate</span> and a <span class='blue'>choice of instalment options</span> to suit you.</span></div></div>";
                        if (!(jQuery('body').find('.slip_tooltip_container').length)) {
                            jQuery(content).appendTo('body').fadeIn('slow');
                        }
                    },
                    mouseleave: function () {
                        // Hover out code
                        jQuery('.slip_tooltip_container').remove();
                    }
                }, '.payment_method_dividebuy');
            </script>
    <?php   
    }elseif( $enable_dsiable == 'no' && !empty(WC()->cart->get_coupons()))
    {

?>
        <script>
            jQuery(document).bind('DOMNodeInserted', function(event) {           
            jQuery(document).find('.payment_method_dividebuy').css('opacity', 0.4);     
                    jQuery('#payment_method_dividebuy').attr('disabled',true);            
                });
            jQuery(document).on('mousemove', ".payment_method_dividebuy" , function(e) {
                mousemovebtn(e, this)
            });
            jQuery(document).on({
                mouseenter: function () {
                    // Hover code
                    var content = "<div class='slip_tooltip_container coupon-box'><div class='slip_tooltip_content'><span class='tooltip-box'>This coupon code cannot be used in conjunction with Interest-Free Credit.</span></div></div>";
                    if (!(jQuery('body').find('.slip_tooltip_container').length)) {
                        jQuery(content).appendTo('body').fadeIn('slow');
                    }
                },
                mouseleave: function () {
                    // Hover out code
                    jQuery('.slip_tooltip_container').remove();
                }
            }, '.payment_method_dividebuy');
        </script>
<?php

    }

   
}
add_action('woocommerce_removed_coupon','removecoupon_c');
function removecoupon_c()
{

    $min_order = get_option('wc_settings_tab_dividebuy_minimum_order_value');
    $dividebuy_grand_total = 0;
        foreach ( WC()->cart->get_cart() as $cart_item_key => $cart_item ){
            $_product   = apply_filters( 'woocommerce_cart_item_product', $cart_item['data'], $cart_item, $cart_item_key );
            $product_id = apply_filters( 'woocommerce_cart_item_product_id', $cart_item['product_id'], $cart_item, $cart_item_key );
            $product_dividebuy_support = get_post_meta( $product_id, 'product_dividebuy_support', true );
            
            if ( $_product && $_product->exists() && $cart_item['quantity'] > 0 && apply_filters( 'woocommerce_cart_item_visible', true, $cart_item, $cart_item_key ) && $product_dividebuy_support == 'enabled' ){
                $dividebuy_grand_total = $dividebuy_grand_total + ($_product->get_price_including_tax() * $cart_item['quantity']);
            }
        }
    $cart_total_amount = $dividebuy_grand_total;
    $enable_dsiable = get_option('dividebuy_enable_coupon_code');
    if($min_order > $cart_total_amount)
    {
      
    ?>
            <script>
                jQuery(document).bind('DOMNodeInserted', function(event) {  

                    jQuery(document).find('.payment_method_dividebuy').css('opacity', 1);

                    jQuery('#payment_method_dividebuy').attr('disabled',true); 


                });
                
                jQuery(document).on('mousemove', ".payment_method_dividebuy" , function(e) {
                    mousemovebtn(e, this)
                });
                jQuery(document).on({
                    mouseenter: function () {
                        // Hover code
                        var content = "<div class='slip_tooltip_container grey'><div class='slip_tooltip_content'><span class='tooltip-box'><span class='blue'>DivideBuy - The easy Interest-Free Checkout.</span> Spend a minimum of <?php echo get_woocommerce_currency_symbol().$order_minimum_value['min_order_amount']; ?> to be eligible to Spread the Cost, Interest-Free using our <span class='blue'>secure platform</span> to fill out our <span class='blue'>60 second application</span>. With us you'll get an <span class='blue'>instant decision. 96% acceptance rate</span> and a <span class='blue'>choice of instalment options</span> to suit you.</span></div></div>";
                        if (!(jQuery('body').find('.slip_tooltip_container').length)) {
                            jQuery(content).appendTo('body').fadeIn('slow');
                        }
                    },
                    mouseleave: function () {
                        // Hover out code
                        jQuery('.slip_tooltip_container').remove();
                    }
                }, '.payment_method_dividebuy');

             

            </script>
    <?php   
    }
    else
    {

    ?>
            <script>            
            jQuery(document).bind('DOMNodeInserted', function(event) {        
                jQuery(document).find('.payment_method_dividebuy').css('opacity', 1);        
                jQuery('#payment_method_dividebuy').attr('disabled',false);       
               
            });
            jQuery(document).on({
                    mouseenter: function () {
                        // Hover code
                         jQuery('.slip_tooltip_container').remove();
                    },
                    mouseleave: function () {
                        // Hover out code
                        jQuery('.slip_tooltip_container').remove();
                    }
                }, '.payment_method_dividebuy');                        
           
                
    
            </script>       
    <?php 

    }



}
?>
